@@ -5,17 +5,19 @@ mergeable_dtype_kinds = frozenset(("i", "u", "m", "M", "S"))
 
 def merge_to_str(*arrs: np.ndarray) -> np.ndarray:
     """
-    Convert one or more arrays of integers, bytes, datetime64, timedelta64 to S(total_bytes + 1).
+    Convert one or more arrays of integers, bytes, datetime64, timedelta64 to \
+    S(total_bytes + (last array is not "S")).
 
     We cannot use arr.byteswap().view("S8") because the trailing zeros get discarded \
-    in np.char.add. Thus, we have to pad with ";".
+    in np.char.add. Thus, we have to pad with ";" unless the last array is bytes.
 
     We copy bytes ("S"), datetime64, timedelta64 verbatim.
     """
     assert len(arrs) > 0
     size = len(arrs[0])
-    str_len = sum(arr.dtype.itemsize for arr in arrs) + 1
-    arena = np.full((size, str_len), ord(";"), dtype=np.uint8)
+    lock = arrs[-1].dtype.kind != "S"
+    str_len = sum(arr.dtype.itemsize for arr in arrs) + lock
+    arena = np.full((size, str_len), ord(";") if lock else 0, dtype=np.uint8)
     pos = 0
     for arr in arrs:
         kind = arr.dtype.kind
